/***** attributeTree.c ********************************************
 * Description: Construct binary tree of gene/attribute pairs. 
 *              Such a tree is attached to each pattern. It
 *              establishes the x-reference between pattern and
 *              gene. 
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Sun Oct 31 10:17:46 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include "medMatch.h"

/* addPatternAttribute: add a node with gene gene at or below pa */
PatternAttributes *addPatternAttribute(Pattern *pattern, PatternAttributes *pa, Gene *gene, int attr){
  int cond; 
  if(pa == NULL){   /* a new pattern attribute has arrived */
    pa = paAlloc();
    pa->gene = gene;
    pa->pattern = pattern;
    pa = setAttribute(pa, attr);
    pa->left = pa->right = NULL;
  }else if((cond = pa->gene->id - gene->id) == 0){          /* gene already present */
    pa = setAttribute(pa, attr);
  }else if(cond < 0){                                        /* less than into left subtree */
    pa->left = addPatternAttribute(pattern, pa->left, gene, attr);
  }else{                                                     /* geater than into right subtree */
    pa->right = addPatternAttribute(pattern, pa->right, gene, attr);
  }
  return pa;
}

/* paAlloc: generate new patternAttribute */
PatternAttributes *paAlloc(void){
  PatternAttributes *pa;
  if((pa = (PatternAttributes *) malloc(sizeof(PatternAttributes))) == NULL){
    printf("ERROR [medMatch]: memory allocation failed.\n");
    assert(0);
  }
  /* switch off all attribute flags */
  pa->isAgn = pa->isAgs = pa->isPs = pa->isA = pa->isPgn = 0;
  return pa;
}


/* setAttribute: set attribute of pa */
PatternAttributes *setAttribute(PatternAttributes *pa, int attr){
  switch(attr){
  case ISAGN:                           /* approved gene name */
    pa->isAgn = 1;
    break;
  case ISAGS:                           /* approved gene symbol */
    pa->isAgs = 1;
    break;
  case ISPS:                            /* previous symbol */
    pa->isPs = 1;
    break;
  case ISA:                             /* alias */
    pa->isA = 1;
    break;
  case ISPGN:                           /* previous gene name */
    pa->isPgn = 1;         
    break;
  default:
    printf("ERROR [medMatch]: unknown attribute type %d\n",attr);
    assert(0);
  }
  return pa;
}


void testTree(PatternAttributes *pa){
  if(pa != NULL){
    printf("gene: %d\n",pa->gene->id);
    testTree(pa->left);
    testTree(pa->right);
  }
}
