/***** interface.c ************************************************
 * Description: Routine for gathering arguments from the command
 *              line.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Sun Jun 20 13:12:10 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "medMatch.h"
#include "stringUtil.h"

Args getArgs(int argc, char *argv[]){
  Args args;
  char c;
  int arg;
 
  args.h = 0;
  args.d = 0;
  args.D = 0;
  args.O = NULL;
  args.o = NULL;
  args.p = "";
  args.t = NULL;
  args.symLen1 = 5;
  args.symLen2 = 3;
  args.symLen3 = 2;
  args.wc1 = 4;
  args.wc2 = 1;
  args.wc3 = 2;
  args.wcAlias = 1;
  args.frac = 0.5;
  args.A = 0;
  args.e = 1;
  args.c = 0;
  arg = 1;
  while(arg < argc){
    c = argv[arg][1];
      switch(c){
      case 'p':                           /* pattern file */
	args.p = argv[++arg];
	args.e = 0;
	break;
      case 't':                           /* text file */
	args.t = argv[++arg];
	break;
      case 'O':                           /* output file for patterns */
	args.O = argv[++arg];
	break;
      case 'o':                           /* output file */
	args.o = argv[++arg];
	break;
      case 'd':
	args.d = 1;
	break;
      case 'D':
	args.D = 1;
	break;
      case 'c':
	args.c = 1;
	break;
      case 's':
	args.symLen1 = atoi(argv[++arg]);
	break;
      case 'u':
	args.symLen2 = atoi(argv[++arg]);
	break;
      case 'r':
	args.symLen3 = atoi(argv[++arg]);
	break;
      case 'w':
	args.wc1 = atoi(argv[++arg]);
	break;
      case 'x':
	args.wc2 = atoi(argv[++arg]);
	break;
      case 'y':
	args.wc3 = atoi(argv[++arg]);
	break;
      case 'z':
	args.frac = atof(argv[++arg]);
	break;
      case 'a':
	args.wcAlias = atoi(argv[++arg]);
	break;
      case 'A':
	args.A = 1;
	break;
      case 'h':                            /* print help */
	args.h = 1;
	break;
      default:
	printf("# unknown argument: %c\n",c);
	args.e = 1;
	return args;
      }
      arg++;
  }
  return args;
}

void printArgsUsage(char *version, Args args){
  printf("medMatch version %s, copyright (c) 2006, Bernhard Haubold & Olaf Bininda-Emonds\n", version);
  printf("\tdistributed under the GNU General Public License\n");
  printf("purpose: create a gene index of Medline; search for Approved Symbol & Previous Symbols is case-sensitive;\n");
  printf("         search for other patterns is not case-sensitive.\n");
  printf("input arguments:\n");                  
  printf("\t-p FILE pattern file in HUGO format (cf. http://www.gene.ucl.ac.uk/nomenclature/data/gdlw_index.html)\n");
  printf("\t[-t FILE text file in Medline format; default: stdin]\n");
  printf("\t[-o FILE output file; default: stdout]\n");
  printf("symbol match arguments:\n");
  printf("\t[-s NUM minimum length of Approved Symbol or Previous Symbols; default: %d]\n",args.symLen1);
  printf("\t[-u NUM minimum length of Approved Symbol or Previous Symbols in conjunction with -x; default: %d]\n",args.symLen2);
  printf("\t[-v NUM minimum length of Approved Symbol or Previous Symbols in conjunction with -y; default: %d]\n",args.symLen3);		  
  printf("word match arguments:\n");									
  printf("\t[-w NUM minimum word count in Approved Name or Previous Names or Aliases; default: %d]\n",args.wc1);
  printf("\t[-x NUM minimum word count in Approved Name or Previous Names or Aliases; default: %d]\n",args.wc2);			
  printf("\t[-y NUM minimum word count in Approved Name or Previous Names or Aliases; default: %d]\n",args.wc3);	  
  printf("\t[-z NUM minimum fraction of Approved Name or Previous Names; default: %1.1f]\n",args.frac);				     
  printf("\t[-a NUM minimum word count in Aliases in conjunction with -z applied to Previous Name; default: %d]\n",args.wcAlias);
  printf("\t[-A match Aliases case-sensitively]\n");				    
  printf("miscellaneous arguments:\n");
  printf("\t[-O FILE output file for patterns (debugging); default: patterns not written to file]\n");
  printf("\t[-d diagnostic output (debugging)\n");
  printf("\t[-D verbose diagnostic output (debugging)]\n");
  printf("\t[-h print this help message]\n");
  printf("\t[-c print match criteria (AN = Approved Name; PN = Previous Names; A = Aliases)\n");
  printf("\t    symbol match\n");
  printf("\t\t1 true if symbol length >= %d\n",args.symLen1);
  printf("\t\t2 true if symbol length >= %d and >= %d words present form AN, PN, or A\n",args.symLen2,args.wc2);
  printf("\t\t3 true if symbol length == %d and >= %d words present form AN, PN, or A\n",args.symLen3,args.wc3);
  printf("\t    word match (can only be true if symbol match is not true)\n");
  printf("\t\t4 true if >= %d words present from AN\n",args.wc1); 
  printf("\t\t5 true if >= %d words present from PN\n",args.wc1);
  printf("\t\t6 true if fraction of words present from AN >= %3.1f\n",args.frac);
  printf("\t\t7 true if fraction of words present from PN >= %3.1f\n",args.frac);
  printf("\t\t8 true if (1 word from AN or PN present) AND (Previous Symbols or Aliases present)]\n");
  printf("limits:\n");
  printf("\tmaximum length of line in text or pattern file: %d\n",MAXLINE);
  printf("\tmaximum number of patterns: %d\n",MAX_PATTERNS);
}
