/***** medMatch.c ***********************************************
 * Description: Match a set of patterns against a medline using a
 *    keyword tree. 
 * Reference: Gusfield, D. (1997). Algorithms on Strings, Trees,
 *    and sequences; Computer Science and Computational Biology.
 *    CUP, p. 52ff.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Fri Aug 27 19:30:32 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <time.h>
#include <assert.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "medMatch.h"
#include "stringUtil.h"
#include "eprintf.h"

int main(int argc, char *argv[]){
  Args args;           /* arguments */
  char *version;       /* program version */
  char **p;            /* pattern data */
  int numChar,i;
  FILE *fp, *outF;
  Gene *genes;
  Pattern **patternTab = NULL;
  Pattern *pp;
  KNode *symbolRoot, *nonSymbolRoot;
  
  version = "1.2";
  setprogname2("medMatch");
  args = getArgs(argc, argv);

  p = NULL;

  if(args.h == 1){
    printArgsUsage(version,args);
    return 0; 
  }else if(args.e == 1){
    printArgsUsage(version,args);
    return -1;
  }
  /* open HUGO file  */
  if((fp = fopen(args.p, "r")) == NULL){
    fprintf(stderr,"ERROR [medMatch]: could not open %s\n.",args.p);
    exit(1);
  }

  /* construct hash table containing patterns */
 /* allocate space for genes */
  if((genes = (Gene *)malloc(MAX_GENES*sizeof(Gene))) == NULL){
    printf("ERROR [medMatch]: memory allocation failed.\n");
    assert(0);
  }
  printf("# Constructing pattern table...");
  fflush(stdout);
  patternTab = constructPatternTable(fp, args, patternTab, &numChar, genes);
  printf("done.\n");
/*   for(i=0;i<MAX_GENES;i++){ */
/*     printf("id: %d\n",genes[i].id); */
/*     printf("wordsAgn: %d\n",genes[i].wordsAgn); */
/*   } */
  /* construct keyword tree */
  printf("# Constructing keyword tree...");
  fflush(stdout);
  if((symbolRoot = createSymbolTree(patternTab, args, numChar)) == NULL){
    return 0;
  }
  if((nonSymbolRoot = createNonSymbolTree(patternTab, args, numChar)) == NULL){
    return 0;
  }
  printf("done.\n");
  if(args.O != NULL){
    if((outF = fopen(args.O, "w")) == NULL){
      fprintf(stderr,"ERROR [medMatch]: could not open %s\n.",args.O);
      exit(1);
    }
    printf("# Patterns written to file %s\n",args.O);
    for(i=0; i<HASHSIZE; i++){
      for(pp = patternTab[i]; pp != NULL; pp = pp->next){
	fprintf(outF,"%s\n",pp->str);
      }
    }
    fclose(outF);
  }
  searchMedline(patternTab,args.t,symbolRoot,nonSymbolRoot,args,genes);
  return 0;
}
