/***** preprocess.c ***********************************************
 * Description: Preprocess keyword tree.
 * Reference: Gusfield, D. (1997). Algorithms on Strings, Trees,
 *    and sequences; Computer Science and Computational Biology.
 *    CUP, p. 58.
 * TO DO: Implement the comment made in exercise 13. on p. 67
 *    of Gusfield (1997).
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Wed Sep  8 10:20:36 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include "medMatch.h"

/*
 * Add failure and output links to tree during a breadth-first
 * traversal.
 * Reference: Gusfield, p. 60.
 */
void preprocessTree(KNode *root){
  KNode *w, *wdash, *v, *vdash;
  KNode *child, *front, *back;
  char x;

  front = back = root;
  front->fail = NULL;
  front->out = NULL;
  while(front != NULL){
    v = front;
    vdash = v->out;
    /* add children of current node to queue */
    for(child=v->child; child != NULL; child=child->sibling){
      child->out = v;
      back->fail = child;
      back = child;
    }
    /* nullify end of queue */
    back->fail = NULL;
    /* move along one step in queue */
    front = front->fail;
    v->fail = v->out = NULL;
    if(v != NULL && vdash != NULL){
      /* set failure link */
      if(v == root){
	v->fail = v;
      }else if(vdash == root){
	v->fail = vdash;
      }else{
	wdash = NULL;
	x = v->c;
	w = vdash->fail;
	while((wdash = findChild(w,x)) == NULL && w != root)
	  w = w->fail;
	if(wdash != NULL && wdash->c == x )
	  v->fail = wdash;
	else
	  v->fail = root;
      }
    }
    /* set output link */
    if(v->fail == NULL)
      ;
    else if(v->fail->p != NULL)
      v->out = v->fail;
    else if(v->fail->out != NULL)
      v->out = v->fail->out;
  }
  root->fail = root;
}
