/***** processPatterns.c ******************************************
 * Description: Process patterns read from the command line.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Mon Oct 25 20:47:24 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "medMatch.h"

/*
 * Get a list of patterns delineated by | and return a pointer
 * to corresponding two-dimensional array of patterns.
 */
char **processPatterns(char *patterns, int *patternSize, int *residues){
  char **p;
  char *word, *newWord = NULL;
  int i, l;

  if((p = (char **)malloc(MAX_PATTERNS*sizeof(char))) == NULL){
    printf("ERROR [interface]: memory allocation failed\n");
    assert(0);
  }
  if((word = (char *)malloc(1000*sizeof(char))) == NULL){
    printf("ERROR [interface]: memory allocation failed\n");
    assert(0);
  }
  i = 0;
  *residues = 0;
  word = strtok(patterns, "|");
  newWord = word;
  l = strlen(word);
  if((p[i] = (char *)malloc((l+1)*sizeof(char))) == NULL){
    printf("ERROR [interface]: memory allocation failed\n");
    assert(0);
  }
  p[i] = word;
  p[i][l] = '\0';
  *residues += strlen(p[i]);
  i++;
  while((word = strtok(NULL, "|")) != NULL){
    l = strlen(word);
    if((p[i] = (char *)malloc((l+1)*sizeof(char))) == NULL){
      printf("ERROR [interface]: memory allocation failed\n");
      assert(0);
    }
    p[i] = strcpy(p[i],word);
    *residues += strlen(p[i]);
    i++;
    newWord = word;
  }
  l = strlen(newWord);
  if((p[i] = (char *)malloc((l+1)*sizeof(char))) == NULL){
    printf("ERROR [interface]: memory allocation failed\n");
    assert(0);
    }
  p[i] = strcpy(p[i],newWord);
  *residues += strlen(p[i]);
  
  *patternSize = i;
  
  /*if((p = (char **)realloc(p, (i+1)*sizeof(char))) == NULL){
    printf("ERROR [interface]: memory reallocation failed\n");
    assert(0);
    }*/

  return p;
}

/*
 * Each line that is read from the file is augmented by a starting and a trailing blank.
 * The purpose of this is to make the search "word sensitive".
 */
char **readPatterns(char *file, char *patterns[], int *patternSize, int *residues){
  char *p[MAX_PATTERNS];
  char *line;
  FILE *fp;
  int pc;
  int len, i=0;
  
  if((line = (char *)malloc(sizeof(char)*MAXLINE))==NULL){
    printf("ERROR [stringUtil]: error in memory allocation\n");
    assert(0);
  }

  patterns = p;
  *patternSize = 0;
  *residues = 0;
  /* open input stream  */
  if(file == "stdin")
    fp = stdin;
  else if((fp = fopen(file, "r")) == NULL){
    fprintf(stderr,"ERROR [stringUtil]: could not open %s\n.",file);
    exit(1);
  }
  pc = 0;
  i=0;
  while((line = fgets(line,MAXLINE,fp)) != NULL && *patternSize < MAX_PATTERNS){
    if(line[0] == '\n')
      continue;
    if(strstr(line,"\n") == NULL)
      len = strlen(line);
    else
      len = strlen(line)-1;
    *residues += len;
    if((p[i] = (char *)malloc((len+1)*sizeof(char))) == NULL){
      printf("ERROR [readPatterns]: memory allocation failed\n");
      assert(0);
    }
    p[i] = strncpy(p[i],line,len);
    /* printf("Pattern loaded: %s\n",p[i]); */
    i++;
  }
  *patternSize = i;
  return patterns;
}
