/***** searchMedline.c ********************************************
 * Description: Search a medline file using a keyword tree.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Mon Sep 13 12:22:31 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>
#include "stringUtil.h"
#include "medMatch.h"
#include "eprintf.h"

int individualWordCount;
int wordCount = 0;    /* counts the number of words found */
int geneCount = 0;    /* counts the number of genes found */
int wordMatch = 0;    /* counts the number of matches based on words as opposed to symbols */
int symbolMatch = 0;  /* counts the number of matches based on symbols */
Gene *geneList;
static int geneIndex[MAX_GENES];
static int symbolIndex[2][MAX_GENES];
static int wordIndex[2][MAX_GENES];
int patternLength;
int first;
char *header;
Args arguments;

int isSymbolMatch(Args args, int symbolLength, int gi);
int isWordMatch(Args args, int gi);

void searchMedline(Pattern *patternTab[], char *fileName, KNode *symbolRoot, KNode *nonSymbolRoot, Args args, Gene *genes){
  FILE *fpi,*fpo;
  char *line;
  WordNode *wordRoot = NULL;
  WordNode *wordNodePool;
  int i, ac;
  int pmidCounter = 0;

  geneList = genes;
  arguments = args;

/*   for(i=0;i<MAX_GENES;i++){  */
/*      if(geneList[i].id != 0)  */
/*        fprintf(fpo,"sym: %s\n",geneList[i].sym);  */
/*    }  */

  line = (char *)malloc((MAXLINE+2) * sizeof(char));
  header = (char *)malloc(200 * sizeof(char));
  wordNodePool = (WordNode *)malloc(MAX_GENES * sizeof(WordNode));

  if(fileName == NULL)
    fpi = stdin;
  else{
    fpi = fopen(fileName, "r");
    if(fpi == NULL){
      fprintf(stderr,"error in searchMedline: could not open %s\n.",fileName);
      assert(0);
    }
  }
  if(args.o)
    fpo = efopen(args.o,"w");
  else
    fpo = stdout;

  while((line = fgets(line, MAXLINE, fpi)) != NULL){
    if(strstr(line,"<MedlineCitation") != NULL || strstr(line,"<PubmedArticle") != NULL)
      pmidCounter = 0;
    if(strstr(line,"<PMID>") != NULL && pmidCounter == 0){
      header = strcpy(header,strtok(line,">")); 
      header = strcpy(header,strtok(NULL,"<"));
      pmidCounter++;
    }else if(strstr(line,"<ArticleTitle>") != NULL){
      wordRoot = searchLine(line, symbolRoot, wordRoot, wordNodePool);
      wordRoot = searchLine(strtolower(line), nonSymbolRoot, wordRoot, wordNodePool);
    }else if(strstr(line,"<AbstractText>") != NULL){
      wordRoot = searchLine(line, symbolRoot, wordRoot, wordNodePool);
      wordRoot = searchLine(strtolower(line), nonSymbolRoot, wordRoot, wordNodePool);
    }else if(strstr(line,"<NameOfSubstance>") != NULL){
      wordRoot = searchLine(line, symbolRoot, wordRoot, wordNodePool);
      wordRoot = searchLine(strtolower(line), nonSymbolRoot, wordRoot, wordNodePool);
    }else if(strstr(line,"<DescriptorName") != NULL){
      wordRoot = searchLine(line, symbolRoot, wordRoot, wordNodePool);
      wordRoot = searchLine(strtolower(line), nonSymbolRoot, wordRoot, wordNodePool);
    }else if(strstr(line,"<QualifierName") != NULL){
      wordRoot = searchLine(line, symbolRoot, wordRoot, wordNodePool);
      wordRoot = searchLine(strtolower(line), nonSymbolRoot, wordRoot, wordNodePool);
    }else if(strstr(line,"</MedlineCitation") != NULL || strstr(line,"</PubmedArticle") != NULL){
      if(wordCount > 0){
	if(args.d){
	  fprintf(fpo,">%s START INDIVIDUAL WORDS\n",header);
	}
	harvestTree(wordRoot);
	if(args.d){
	  fprintf(fpo,">%s END INDIVIDUAL WORDS\n",header);
	}
	wordCount = 0;
	wordRoot = NULL;
	ac = getApprovedCount(args);
/* 	fprintf(fpo,"symbolMatch: %d ",symbolMatch); */
/* 	fprintf(fpo,"wordMatch: %d\n",wordMatch); */
	if(ac > 0 &&(symbolMatch > 0 || wordMatch <= MAXWORDGENES)){
	  fprintf(fpo,"%s\t",header);
	  for(i=0;i<symbolMatch;i++){
	    fprintf(fpo,"%s",geneList[symbolIndex[0][i]].sym);
	    if(args.c)
	      fprintf(fpo,":%d ",symbolIndex[1][i]);
	    else
	      fprintf(fpo," ");
/* 	    fprintf(fpo,"%s:%d ",geneList[symbolIndex[0][i]].sym,symbolIndex[1][i]); */
	    if(args.d || args.D){
	      fprintf(fpo,"\tSymbol length: %d\t", geneList[symbolIndex[0][i]].lengthAgs);
	      fprintf(fpo,"Old Symbol length: %d\t", geneList[symbolIndex[0][i]].lengthPs);
	      fprintf(fpo,"Name: %d\t", geneList[symbolIndex[0][i]].countAgn);
	      fprintf(fpo,"Old Name: %d\t", geneList[symbolIndex[0][i]].countPgn);
	      fprintf(fpo,"Alias: %d\t", geneList[symbolIndex[0][i]].countAlias);
	      fprintf(fpo,"Words Name: %d\t",geneList[symbolIndex[0][i]].wordsAgn); 
	      fprintf(fpo,"Words Old Name: %d\n",geneList[symbolIndex[0][i]].wordsPgn);
	    }
	  }
	  symbolMatch = 0;
	  if(wordMatch <= MAXWORDGENES){
	    for(i=0;i<wordMatch;i++){
	      fprintf(fpo,"%s",geneList[wordIndex[0][i]].sym);
	      if(args.c)
		fprintf(fpo,":%d ",wordIndex[1][i]);
	      else
		fprintf(fpo," ");
/* 	      fprintf(fpo,"%s:%d ",geneList[wordIndex[0][i]].sym,wordIndex[1][i]); */
	      if(args.d || args.D){
		fprintf(fpo,"\tSymbol length: %d\t", geneList[wordIndex[0][i]].lengthAgs);
		fprintf(fpo,"Old Symbol length: %d\t", geneList[wordIndex[0][i]].lengthPs);
		fprintf(fpo,"Name: %d\t", geneList[wordIndex[0][i]].countAgn);
		fprintf(fpo,"Old Name: %d\t", geneList[wordIndex[0][i]].countPgn);
		fprintf(fpo,"Alias: %d\t", geneList[wordIndex[0][i]].countAlias);
		fprintf(fpo,"Words Name: %d\t",geneList[wordIndex[0][i]].wordsAgn);
		fprintf(fpo,"Words Old Name: %d\n",geneList[wordIndex[0][i]].wordsPgn);
	      }
	    }
	  }
	  fprintf(fpo,"\n");
	}
	wordMatch = 0;
	resetGenes(geneCount,geneIndex); 
	geneCount = 0;
      }
    }
  }
}


void resetGenes(int count,int *ind){
  int i;

  for(i=0;i<count;i++){
    geneList[ind[i]].isNew = 1;
    geneList[ind[i]].wordsAgn = 0;
    geneList[ind[i]].wordsPgn = 0;
    geneList[ind[i]].countAgn = 0;
    geneList[ind[i]].countPgn = 0;
    geneList[ind[i]].countAlias = 0;
    geneList[ind[i]].lengthAgs = 0;
    geneList[ind[i]].lengthPs = 0;
  }
}
/* go through gene list, count the number of approved genes, and place their ids in the geneIndex; 
 */
int getApprovedCount(Args args){
  int i, gi, l;

  for(i=0; i<geneCount; i++){
    gi = geneIndex[i];

    l = (geneList[gi].lengthPs > geneList[gi].lengthAgs)  ? geneList[gi].lengthPs : geneList[gi].lengthAgs;

/*     printf("symbol: %s; wordsAgn: %d, countAgn: %d",geneList[gi].sym,geneList[gi].wordsAgn,geneList[gi].countAgn); */
/*     printf(" (int)(geneList[gi].wordsAgn*args.frac+1): %d",(int)(geneList[gi].wordsAgn*args.frac+1)); */
/*     if(geneList[gi].countAgn >= (int)(geneList[gi].wordsAgn*args.frac+1)) */
/*       printf(" word match\n"); */
/*     else */
/*       printf(" no word match\n"); */

    if((symbolIndex[1][symbolMatch] = isSymbolMatch(args, l, gi))){
       symbolIndex[0][symbolMatch++] = gi;
    }else if((wordIndex[1][wordMatch] = isWordMatch(args, gi))){
      wordIndex[0][wordMatch++] = gi;
    }
  }
  return wordMatch + symbolMatch;
}

int isSymbolMatch(Args args, int symbolLength, int gi){

  if(symbolLength >= args.symLen1){
    return 1;
  }else if(symbolLength >= args.symLen2){
    if(geneList[gi].countAgn + geneList[gi].countPgn + geneList[gi].countAlias >= args.wc2){
      return 2;
    }
  }else if(symbolLength == args.symLen3){
    if(geneList[gi].countAgn + geneList[gi].countPgn + geneList[gi].countAlias >= args.wc3){
      return 3;
    }
  }
  return 0;
}

/* Check word matches */
int isWordMatch(Args args, int gi){
  if(geneList[gi].countAgn >= args.wc1){                                                                                            /* word match */
    return 4;
  }else if(geneList[gi].countPgn >= args.wc1){                                                                                       /* word match */
    return 5;
  }
/*   else if(geneList[gi].countAgn >= (int)(geneList[gi].wordsAgn*args.frac+1)){                                                       /\* word match *\/ */
/*     return 6; */
/*   }else if(geneList[gi].countPgn >= (int)(geneList[gi].wordsPgn*args.frac+1)){                                                       /\* word match *\/ */
/*     return 7; */
/*   } */
  else if((geneList[gi].countAgn > 1 || geneList[gi].countPgn > 1) && (geneList[gi].lengthPs > 1 || geneList[gi].countAlias >= 1)){ /* word match */
    return 8;
  }
  
  return 0;
  
}

void harvestTree(WordNode *p){

  if(p != NULL){
    harvestTree(p->left);
    
    individualWordCount = p->count;                        /* global variable */
    if(arguments.d || arguments.D){
      printf("!!!%s:%d\n",p->pattern->str,p->count);
    }
    patternLength = strlen(p->pattern->str);               /* global variable */
    harvestPatternAttributes(p->pattern->pa);

    harvestTree(p->right);
  }
}

/* traverse tree of pattern attributes, note the indices of the corresponding genes in geneIndex
 * and do gene accounting in the gene contained in geneList
 */
void harvestPatternAttributes(PatternAttributes *pa){
  int gi;
  if(pa != NULL){
    harvestPatternAttributes(pa->left);
    if(arguments.D){
      printf("%s; individualWordCount: %d; patternLength: %d",pa->gene->sym,individualWordCount,patternLength);
      if(pa->isAgs || pa->isPs)
	printf(" SymbolM");
      if(pa->isAgn || pa->isPgn)
	printf(" WordM");
      if(pa->isA)
	printf(" AliasM");
      printf("\n");
    }
    if(individualWordCount <= MAX_WORD_COUNT || pa->isAgs || pa->isPs){
      gi = pa->gene->id;
      if(geneList[gi].isNew){
	geneIndex[geneCount++] = gi;
	geneList[gi].isNew = 0;
      }
      if(pa->isAgs){
	geneList[gi].lengthAgs = patternLength;
      }
      if(pa->isAgn){
	geneList[gi].countAgn += pa->pattern->scoreAgn;
	/* geneList[gi].countAgn++; */
      }
      if(pa->isPgn){
	geneList[gi].countPgn += pa->pattern->scorePgn;
	/* geneList[gi].countPgn++; */
      }
      if(pa->isPs){
	if(patternLength > geneList[gi].lengthPs)
	  geneList[gi].lengthPs = patternLength;
      }
      if(pa->isA){
	geneList[gi].countAlias++;
      }
    }
    harvestPatternAttributes(pa->right);
  } 
}

WordNode *addWord(WordNode *p, Pattern *pattern, WordNode *wordNodePool){
  int cond;

  if(p == NULL){                       /* a new word has arrived */
    p = &wordNodePool[wordCount++];    /* make a new node */
    p->pattern = pattern;
    p->count = 1;
    p->left = p->right = NULL;
  }else if((cond = strcmp(pattern->str, p->pattern->str)) == 0)
    p->count++;       /* repeated word */
  else if(cond < 0)   /* less than into left subtree */
    p->left = addWord(p->left, pattern, wordNodePool);
  else                /* greater than into right subtree */
    p->right = addWord(p->right, pattern, wordNodePool);
  return p;
}

/*
 * Search one line of text for words.
 */
WordNode *searchLine(char *text, KNode *root, WordNode *wordRoot, WordNode *wordNodePool){
  int l, c, length;
  KNode *w, *wdash, *np;

  w = root;
  l = 0;
  c = 0;
  length = strlen(text);

  while(c < length){
    while((wdash = findChild(w,text[c])) != NULL){
      w = wdash;
      if(wdash->p != NULL){ 
	np = wdash;
	while(np != NULL){
	  wordRoot = addWord(wordRoot,np->p,wordNodePool);
	  np = np->out;
	}
      }
      c++;
    }
    if(w == root){
      c++;
    }else{
      w = w->fail;
    }
  }
  return wordRoot;
}
