/***** sequenceData.c *********************************************
 * Description: Collection of routines for reading and 
 * manipulating sequence data.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Sun Jun  6 10:34:31 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "sequenceData.h"
#include "stringUtil.h"

/* reverse and complement a sequence */
Sequence *revcomp(Sequence *seq){
  int i,j,n;
  char c;
  Sequence *newSeq;

  newSeq = NULL;
  
  n = strlen(seq->seq);
  newSeq->seq = (char *) malloc(n*sizeof(char));

  newSeq->id = seq->id; 
  j=0;
  for(i = n-1; i >= 0; i--){
    c = seq->seq[i];
    switch(c){
    case 'a':
      newSeq->seq[j++] ='t';
      break;
    case 'c':
      newSeq->seq[j++] ='g';
      break;
    case 'g':
      newSeq->seq[j++] ='c';
      break;
    case 't':
      newSeq->seq[j++] ='a';
      break;
    case 'A':
      newSeq->seq[j++] ='T';
      break;
    case 'C':
      newSeq->seq[j++] ='G';
      break;
    case 'G':
      newSeq->seq[j++] ='C';
      break;
    case 'T':
      newSeq->seq[j++] ='A';
      break;
    default:
      newSeq->seq[j++] =c;
      break;
    }
  }
  return newSeq;
  
}

/* read FASTA file and store data; return size of data set */
Sequence *readFasta(char *file, Sequence *data, int *size, int *residues){
    FILE *fp;
    char *line;
    int first, l, numentry, maxseqlen, seqlen;
    int *lengthptr;
    Sequence *od;

    line = (char *)malloc((SEQLINE+2)*sizeof(char));

    /* open input file NB: this has got to be an external file, otherwise the rewind fails */
    fp = fopen(file, "r");
    if(fp == NULL){
	fprintf(stderr,"error in readFasta: could not open %s\n.",file);
	exit(1);
    }
    
    /* count number of entries and determine length of longest sequence */
    numentry = 0;
    maxseqlen = 0;
    seqlen = 0;
    *residues = 0;
    while((line=fgets(line,SEQLINE+2,fp)) != NULL){
	if(strlen(line)>SEQLINE){
	    printf("error in readFasta: cannot deal with lines longer than %d bp.\n",SEQLINE);
	    printf("  change the SEQLINE parameter in file sequenceData.h and recompile.\n");
	    exit(2);
	}
	if(strstr(line,">") != NULL){
	    numentry++;
	    if(seqlen > maxseqlen)
		maxseqlen = seqlen;
	    seqlen = 0;
	}else{
	    l = strlen(line) - 1;
	    seqlen += l; 
	    *residues += l;
	}
    }
    *size = numentry;
    /* allocate memory */
    data = (Sequence *)malloc(numentry*sizeof(Sequence));
    lengthptr = (int *)malloc(numentry*sizeof(int));
    line = (char *)malloc((SEQLINE+2)*sizeof(char));
    od = data;
    rewind(fp);
    /* Determine the length of each input sequence */
    lengthptr--;
    while((line=fgets(line,SEQLINE+2,fp)) != NULL){
	if(strstr(line,">") != NULL){
	    lengthptr++;
	    *lengthptr=0;
	}else{
	    l = strlen(line) - 1;
	    *lengthptr += l;
	}
    }

    line = (char *)malloc((SEQLINE+2)*sizeof(char));
    rewind(fp);

    first = 1;
    lengthptr -= numentry;
    data--;
    while((line=fgets(line,SEQLINE,fp)) != NULL){
	line = chomp(line);
	if(strstr(line,">") != NULL){           /* found new entry */
	    if(first == 0){
		data->seq -= *lengthptr;        /* readjust sequence pointer position */
	    }else{
		first = 0;
	    }
	    lengthptr++;
	    data++;
	    data->seq = (char *)malloc(*lengthptr * sizeof(char));
	    data->id = (char *)malloc((strlen(line)+1)*sizeof(char));
	    strcpy(data->id,line);
	}else{
	    strcpy(data->seq,line);
	    data->seq += strlen(line);
	}
    }
    /* readjust last sequence pointer */
    data->seq -= *lengthptr;
    free(line);
    return od;
}
