/***** stringUtil.c **********************************************
 * Description: Collection of string handling functions.
 * Author: Bernhard Haubold, bernhard.haubold@fh-weihenstephan.de
 * File created on Sun Jun  6 10:02:16 2004.
 *
 * This file is part of medMatch.
 *
 *   medMatch is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   medMatch is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with medMatch; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 ****************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <ctype.h>
#include "stringUtil.h"

/* chomp: remove carriage return from string */
char *chomp(char *line){
    char *ol;
    
    ol = line;
    line = strpbrk(line,"\n");
    *line = '\0';
    return ol;
}

/* strtolower: convert a string to lower case */
char *strtolower(char *s){
  int i,l;
  l = strlen(s);
  for(i=0;i<l;i++)
    if(isupper(s[i])) /* upper case letter changed to lower case */
      s[i] = tolower(s[i]);
  return s;  
}

/* strdup: make a duplicate of s */
char *strdup2(char *s){
  char *p;
  p = (char *) malloc(strlen(s)+1);    /* +1 for '\0' */
  if (p != NULL)
    p = strcpy(p,s);
  return p;
}

void split(char *line, char *splitC, char *splitArray[], int *arrayLen){
  char *line2, c;
  int i, j, start, end;

  start = end = 0;
  *arrayLen = 0;
  line2 = (char *) malloc(strlen(line)+1);    /* +1 for '\0' */
  if (line2 != NULL)
    line2 = strcpy(line2,line);
  i = 0;
  /* count number of fields */
  while((c = line2[end++]) != '\0'){
    if(c == *splitC){
      splitArray[*arrayLen] = (char *) malloc((end - start)*sizeof(char));
      i=0;
      for(j=start;j<end-1;j++){
	splitArray[*arrayLen][i++] = line2[j];
      }
      splitArray[*arrayLen][i] = '\0';
      start = end;
      (*arrayLen)++;
    }
  }
  i=0; 
  splitArray[*arrayLen] = (char *) malloc((end-start+1)*sizeof(char));
  for(j=start;j<end;j++){
    splitArray[*arrayLen][i++] = line2[j];
  }
  splitArray[*arrayLen][i] = '\0';
  (*arrayLen)++;
}

/* remove blanks from string */
char *rmBlanks(char *str){
  int l, i, j;
  l = strlen(str);
  j = 0;
  for(i=0;i<l;i++){
    if(str[i] != ' ')
      str[j++] = str[i];
  }
  str[j] = '\0';
  return str;
}

/* remove leading and trailing non-alphanumerical characters */
char *cleanWordEdges(char *word){
  int l1, l2;
  char *word2 = NULL;

  /* printf("cwe before: %s", word); */

  /* remove leading non-alphanumericals */
  while(!isalnum(*word) && *word != '\0'){
    word++;
  }


  /* remove trailing non-alphanumericals */
  l1 = strlen(word);
  l2 = l1;
  while(l2 > 0 && !isalnum(word[l2-1])){
    l2--;
  }
  if(l2 < l1){
    if((word2 = (char *)malloc((l2+1)*sizeof(char))) == NULL){
      printf("ERROR [stringUtil]: memory allocation failed.\n");
      assert(0);
    }
    word2 = strncpy(word2,word,l2);
    /* printf(" cwe after: %s\n",word2); */
    return word2; 
  }else{
    /* printf(" cwe after: %s\n",word); */
    return word;
  }
  
}

/* remove all () and [] from word; check for resulting double hyphenation */
char *cleanWord(char *word){
  int i, j, l1, h;

  /* printf("cw before: %s", word); */

  /* get word length */
  l1 = strlen(word);
  j=0;
  h=0;
  for(i=0;i<l1;i++){
    if(!(word[i] == '[' || word[i] == ']' || word[i] == '(' || word[i] == ')')){
      if(word[i] == '-'){
	h++;
      }else{
	h=0;
      }
      if(h < 2)
	word[j++] = word[i];
    }
  }
  word[j] = '\0';
  return word;
}

/* hash: form hash value for string s */
unsigned hash(char *s){
  unsigned hashval;
  for(hashval = 0; *s != '\0'; s++)
    hashval = *s +31 * hashval;

  return hashval % HASHSIZE;
}
